(eval-when-compile
  (require 'cl))       ; flet in org-babel-spec-to-string
(require 'org-install) ; needs Version 7.7
(require 'org)

;; Extract Initial Comment Text and State from Org Buffer
;; 
;;    The initial comment text ranges from either the most recent headline
;;    at the point after the stars, the beginning of the line after the
;;    =#+end_src= of the most recent code block, or the beginning of the
;;    buffer, whichever is later, through the line before the source
;;    block.
;;    
;;    The code to extract this is given below. It replaces lines 344
;;    through 357 of =ob-tangle.el= from org-mode version 7.7 in the
;;    function =org-babel-tangle-collect-blocks=.

(defun org-babel-tangle-collect-blocks (&optional language)
  "Collect source blocks in the current Org-mode file.
Return an association list of source-code block specifications of
the form used by `org-babel-spec-to-string' grouped by language.
Optional argument LANG can be used to limit the collected source
code blocks by language."
  (let ((block-counter 1) (current-heading "") blocks)
    (org-babel-map-src-blocks (buffer-file-name)
      ((lambda (new-heading)
	 (if (not (string= new-heading current-heading))
	     (progn
	       (setq block-counter 1)
	       (setq current-heading new-heading))
	   (setq block-counter (+ 1 block-counter))))
       (replace-regexp-in-string "[ \t]" "-"
				 (condition-case nil
				     (nth 4 (org-heading-components))
				   (error (buffer-file-name)))))
      (let* ((start-line (save-restriction (widen)
					   (+ 1 (line-number-at-pos (point)))))
	     (file (buffer-file-name))
	     (info (org-babel-get-src-block-info 'light))
	     (src-lang (nth 0 info)))
        (unless (string= (cdr (assoc :tangle (nth 2 info))) "no")
          (unless (and language (not (string= language src-lang)))
	    (let* ((info (org-babel-get-src-block-info))
		   (params (nth 2 info))
		   (link ((lambda (link)
			    (and (string-match org-bracket-link-regexp link)
				 (match-string 1 link)))
			  (org-babel-clean-text-properties
			   (org-store-link nil))))
		   (source-name
		    (intern (or (nth 4 info)
				(format "%s:%d"
					current-heading block-counter))))
		   (expand-cmd
		    (intern (concat "org-babel-expand-body:" src-lang)))
		   (assignments-cmd
		    (intern (concat "org-babel-variable-assignments:" src-lang)))
		   (body
		    ((lambda (body) ;; run the tangle-body-hook
		       (with-temp-buffer
			 (insert body)
			 (run-hooks 'org-babel-tangle-body-hook)
			 (buffer-string)))
		     ((lambda (body) ;; expand the body in language specific manner
			(if (assoc :no-expand params)
			    body
			  (if (fboundp expand-cmd)
			      (funcall expand-cmd body params)
			    (org-babel-expand-body:generic
			     body params
			     (and (fboundp assignments-cmd)
				  (funcall assignments-cmd params))))))
		      (if (and (cdr (assoc :noweb params)) ;; expand noweb refs
			       (let ((nowebs (split-string
					      (cdr (assoc :noweb params)))))
				 (or (member "yes" nowebs)
				     (member "tangle" nowebs))))
			  (org-babel-expand-noweb-references info)
			(nth 1 info)))))
                   (comment
                    (when (or (string= "both" (cdr (assoc :comments params)))
                              (string= "org" (cdr (assoc :comments params))))
                      (let* ((prev-heading
                              (condition-case nil
                                  (save-excursion
                                    (org-back-to-heading t) ; sets match data
                                    (match-end 0))
                                (error (point-min))))
                             (end-of-prev-src-block
                              (save-excursion
                                (if (null (re-search-backward
                                           org-babel-src-block-regexp nil t))
                                    (point-min)
                                  (goto-char (match-end 0))
                                  (forward-line 1)
                                  (point))))
                             (comment-start
                              (max prev-heading end-of-prev-src-block))
                             (comment-end
                              (save-excursion
                                (forward-line 0)
                                (point)))
                             (state
                              (list (cons 'org-drawers
                                          org-drawers)
                                    (cons 'after-heading
                                          (= comment-start prev-heading))
                                    (cons 'first-line
                                          (= comment-start (point-min))))))
                        (org-babel-process-comment-text
                         (buffer-substring comment-start comment-end) state))))
		   by-lang)
	      ;; add the spec for this block to blocks under it's language
	      (setq by-lang (cdr (assoc src-lang blocks)))
	      (setq blocks (delq (assoc src-lang blocks) blocks))
	      (setq blocks (cons
			    (cons src-lang
				  (cons (list start-line file link
					      source-name params body comment)
					by-lang)) blocks)))))))
    ;; ensure blocks in the correct order
    (setq blocks
          (mapcar
	   (lambda (by-lang) (cons (car by-lang) (reverse (cdr by-lang))))
	   blocks))
    blocks))

;; Adjust =org-babel-spec-to-string
;;
;;   The commment block collected by the original code in
;;   =org-babel-tangle-collect-blocks= is further processed in
;;   =org-babel-spec-to-string= to trim leading and trailing whitespace
;;   from string. This was needed because spaces after a source block
;;   were included in the comment. In the revised code, however, this
;;   space trimming is handled during text transformation, except for
;;   removing trailing newlines. (Note: trailing /spaces/ are not
;;   removed to allow more flexibility in comment processing.) Hence,
;;   =org-babel-spec-to-string= needs to be slightly adjusted.

(defun org-babel-spec-to-string (spec)
  "Insert SPEC into the current file.
Insert the source-code specified by SPEC into the current
source code file.  This function uses `comment-region' which
assumes that the appropriate major-mode is set.  SPEC has the
form

  (start-line file link source-name params body comment)"
  (let* ((start-line (nth 0 spec))
	 (file (nth 1 spec))
	 (link (org-link-escape (nth 2 spec)))
	 (source-name (nth 3 spec))
	 (body (nth 5 spec))
	 (comment (nth 6 spec))
	 (comments (cdr (assoc :comments (nth 4 spec))))
	 (padline (not (string= "no" (cdr (assoc :padline (nth 4 spec))))))
	 (link-p (or (string= comments "both") (string= comments "link")
		     (string= comments "yes") (string= comments "noweb")))
	 (link-data (mapcar (lambda (el)
			      (cons (symbol-name el)
				    ((lambda (le)
				       (if (stringp le) le (format "%S" le)))
				     (eval el))))
			    '(start-line file link source-name))))
    (flet ((insert-comment (text)
            (let ((text (org-babel-chomp text "[\f\t\n\r\v]")))
	      (when (and comments (not (string= comments "no"))
			 (> (length text) 0))
		(when padline (insert "\n"))
		(comment-region (point) (progn (insert text) (point)))
		(end-of-line nil) (insert "\n")))))
      (when comment (insert-comment comment))
      (when link-p
	(insert-comment
	 (org-fill-template org-babel-tangle-comment-format-beg link-data)))
      (when padline (insert "\n"))
      (insert
       (format
	"%s\n"
	(replace-regexp-in-string
	 "^," ""
	 (org-babel-trim body (if org-src-preserve-indentation "[\f\n\r\v]")))))
      (when link-p
	(insert-comment
	 (org-fill-template org-babel-tangle-comment-format-end link-data))))))

;; Process Comment Text Through Sequence of Transforms   
;; 
;;    At the end of the revised [[preferred-fix][comment collection code]],
;;    the comment text is passed to
;;    =org-babel-process-comment-text= which
;;    applies a sequence of transformation functions.
;;    The list of transformation functions is stored in a customizable
;;    variable described [[Define Customization Variable for Transforms][below]]. Several predefined transformations are
;;    given [[Define A Collection of Transform Functions][below]] as well.

(defun org-babel-process-comment-text (text &optional state)
  "Apply list of transforms to comment TEXT assuming bindings in alist STATE.
Returns the modified text string, which may have text properties.
See `org-babel-comment-processing-functions' for the transforms to be
applied and details on the allowed keys in the STATE alist."
  (let ((funcs org-babel-comment-processing-functions))
    (with-temp-buffer
      (insert text)
      (let ((org-drawers
             (or (cdr (assoc 'org-drawers state))
                 org-drawers))
            (after-heading
             (cdr (assoc 'after-heading state)))
            (first-line
             (cdr (assoc 'first-line state))))
        (while funcs
          (goto-char (point-min))
          (funcall (car funcs))
          (setq funcs (cdr funcs))))
      (buffer-substring (point-min) (point-max)))))

;; Define Customization Variable for Transforms
;; 
;;    A list of nullary functions applied in order to
;;    the comment text. The text is inserted
;;    in a temporary buffer, so these functions can use
;;    the entire Emacs library for operating on buffer text.

(defcustom org-babel-comment-processing-functions
  '(org-babel-comment-delete-file-variables-line
    org-babel-comment-delete-org-control-lines
    org-babel-comment-delete-drawers
    org-babel-comment-trim-blank-lines
    org-babel-comment-trim-indent-prefix)
  "List of functions to transform source-block comment text before insertion.
Each function will be called with no arguments with point at the
beginning of a buffer containing only the comment text. Each
function can modify the text at will and leave point anywhere,
but it should *not* modify the narrowing state of the buffer.
Several dynamic state variables are set prior to execution that
each function can reference. These currently include:

   + org-drawers:   names of drawers in the original org buffer.
   + from-heading:  t if comment starts at an org heading line,
                    nil otherwise.
   + first-line:    t if initial comment starts on first line
                    of the original org buffer, nil otherwise.

If a function changes the value of these state variables, the new
value will be seen by all following functions in the list, but
this is not generally recommended.

The functions in this list are called *in order*, and this order
can influence the form of the resulting comment text."
  :group 'org-babel
  :type 'list)

;; Define A Collection of Transform Functions
;; 
;;    An advantage of this design is that transformation of the
;;    comments is modular and customizable. We can include in
;;    =ob-tangle.el= a collection of pre-defined transforms.
;;    The default processing stream in =org-babel-comment-processing-functions=
;;    is as follows:
;; 
;;    1. Delete a file variables if on the first line of the buffer.
;;    2. Delete all drawers and their contents.
;;    3. Delete all org control lines from the comment text.
;;    4. Trim blank lines from the beginning and end.
;;    5. Reindent the text by removing the longest common leading 
;;       string of spaces.
;; 
;;    These and several other useful transforms are given below
;;    (e.g., deleting drawer delimiters but not contents)..
;;    It is easy to define new transforms; any function that
;;    operates on text in the current buffer beginning at point-min
;;    will work.

(defun org-babel-comment-delete-file-variables-line ()
  "Delete file variables comment line if at beginning of buffer.
This only checks the first line of the buffer, and so should be
placed first (or at least early enough) in the list
`org-babel-comment-processing-functions' to ensure that the no
other text has been inserted earlier."
  (when (and first-line
             (looking-at ; file-variables line
              "^#[ \t]*-\\*-.*:.*;[ \t]**-\\*-[ \t]*$"))
    (let ((kill-whole-line t))
      (kill-line))))

(defun org-babel-comment-delete-org-control-lines ()
  "Remove all org #+ control lines from comment."
  (let ((control-regexp "^[ \t]*#\\+.*$"))
    (delete-matching-lines control-regexp)))

(defun org-babel-comment-delete-org-in-buffer-settings ()
  "Remove all org #+ in-buffer setting lines, leaving other control lines.
In-buffer setting lines begin with #+ and have all caps keyword
names."
  (let ((setting-regexp "^#\\+[ \t]*[A-Z_]+:.*$"))
    (delete-matching-lines setting-regexp)))

(defun org-babel-comment-delete-drawers ()
  "Delete drawer delimiters and contents from comment.
Drawer names are restricted to those in the `org-drawers' state."
  (let ((drawer-start-regexp
         (format "^[ \t]*:\\(?:%s\\):[ \t]*$"
                 (mapconcat 'identity
                            org-drawers
                            "\\|")))
        (drawer-end-regexp "^[ \t]*:END:[ \t]*$"))
    (while (re-search-forward drawer-start-regexp nil t)
      (let ((beg (save-excursion
                   (forward-line 0)
                   (point)))
            (end (save-excursion
                   (re-search-forward drawer-end-regexp nil t)
                   (forward-line 1)
                   (point))))
        (goto-char end)
        (delete-region beg end)))))

(defun org-babel-comment-delete-drawer-delimiters ()
  "Delete drawer delimiters from comment leaving content.
Drawer names are restricted to those given by the `org-drawers'
state."
  (let ((drawer-delim-regexp
         (format "^[ \t]*:\\(?:%s\\)"
                 (mapconcat 'identity
                            (cons "END" org-drawers)
                            "\\|"))))
    (delete-matching-lines drawer-delim-regexp)))

(defun org-babel-comment-trim-blank-lines ()
  "Trim whitespace-only lines from beginning and end of text."
  (while (and (looking-at "^[ \t\f]*$")
              (< (point) (point-max)))
    (forward-line 1))
  (delete-region (point-min) (point))
  (when (< (point) (point-max))
    (goto-char (point-max))
    (let ((last-point (point)))
      (forward-line 0)
      (while (and (looking-at "^[ \t\f]*$")
                  (> (point) (point-min)))
        (setq last-point (point))
        (forward-line -1))
      (delete-region last-point (point-max)))))

(defun org-babel-comment-trim-indent-prefix ()
  "Remove longest common leading prefix of spaces from each line of TEXT.
Prefix is computed from the initial whitespace on each line with
tabs converted to spaces, preserving indentation."
  (let* ((common-indent nil)
         (common-length (1+ (- (point-max) (point-min))))
         (current-indent "")                   ; enter first loop
         (current-length common-length))       ; skip first assignment
    (goto-char (point-min))
    (while current-indent
      (when (< current-length common-length)
        (setq common-indent current-indent
              common-length current-length))     
      (setq current-indent
            (let* ((found (re-search-forward "^\\([ \t]*\\)\\S-" nil t))
                   (bol (match-beginning 0))
                   (eos (match-end 1))
                   (space-str (match-string 1))
                   (indent-tabs-mode nil))
              (cond
               ((not found)
                nil)
               ((not (string-match "\t" space-str))
                space-str)
               (t                       ; detabify indent string
                (goto-char eos)
                (let ((col (current-column)))
                  (delete-region bol eos)
                  (indent-to col))
                (buffer-substring-no-properties bol (point))))))
      (setq current-length (length current-indent)))
    (when (and common-indent (> common-length 0))
      (let ((indent-re (concat "^" common-indent)))
        (goto-char (point-min))
        (while (re-search-forward indent-re nil t)
          (replace-match "" nil nil))))))

;; This kind of customization offers some nice possibilities,
;; including controlling indentation, eliminating or
;; transforming org markup, eliminating trailing whitespace, and
;; automating specialized comment formatting (e.g., javadoc). As
;; an additional illustration, consider the transform
;; =org-babel-comment-restrict-comment-range=
;; below. The idea is that it is sometimes useful to select
;; from the text under a headline a /part/ of the text for
;; the comment. We want some org markup that will not affect
;; either the export or the structure of the org file itself.
;; To do this, we use the fact that =#+=\textvisiblespace
;; lines are not exported.[fn:2] So, we can /de facto/ use the
;; =#+ TANGLE:= construct to control various aspects of tangling.
;; Here, we use the =#+ TANGLE: start-comment= and
;; =#+ TANGLE: end-comment= to delimit the comment text.
;; (This function needs to come earlier in the function list than the
;; functions that eliminate org control lines. It is sufficient to
;; prepend it to that list.) This is used in the current file,
;; for example.

(defun org-babel-comment-restrict-comment-range ()
  "Remove all comment text outside start-comment and end-comment delimiters.
Comment delimiters are #+TANGLE lines with respective keywords
start-comment and end-comment. THE #+TANGLE lines are also
deleted. To be effective, this function should be positioned in
the list `org-babel-comment-processing-functions' before any
functions that remove org control lines or process other
co-occuring attributes of #+TANGLE lines."
  (when (re-search-forward "^[ \t]*#\\+[ \t]*TANGLE:.*start-comment.*$" nil t)
    (forward-line 1)
    (delete-region (point-min) (point)))
  (when (re-search-forward "^[ \t]*#\\+[ \t]*TANGLE:.*end-comment.*$" nil t)
    (forward-line 0)
    (delete-region (point) (point-max))))
